
        LIST    p=16F628 ; PIC16F628 is the target processor

    #include "P16F628.INC" ; Include header file
    #define E 0 ;pick any port A pins (0-4)
    #define RS 2

    cblock 0x20   ; Temporary storage
       outbyte
		 config		;bit7:line-wrap,bit6:autoscroll,bit5:Row, bit3=1,bit2=1,bit1=Underline,bit0=Block
		 cursor		;bits0-3:column,bit4:newline test
		 test
		 nlcount
		 set_row
		 set_col
       time1
       time2
    endc

    __CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _LVP_OFF & _DATA_CP_OFF & _BODEN_OFF 
        org   0

        goto    start

;********************************************************************
;              LCD Routines 
;********************************************************************
;sends high 4 bits of variable 'char' to lcd
nybout  movlw b'11110000'   ;high nybble bitmask
    andwf outbyte,w     ;set top nybble
    iorwf PORTB,f       ;high bits set, now low bits must be set
    movlw b'00001111'   ;high nybble bitmask
    iorwf outbyte,w     ;set top nybble
    andwf PORTB,f       ;low bits set
    bsf PORTA,E
    call qdelay         ;this is necessary, even for a 4MHz clock
    bcf PORTA,E         ;clock it
    retlw 1

;send a character/command in w to lcd
lcdcmd  bcf PORTA,RS        ;RS low for commands
    goto skiprs     ;doing it this way allows the pin to be used for something else
lcdchar bsf PORTA,RS        ;RS high for characters
skiprs  movwf outbyte
    call nybout
    swapf outbyte,f     ;put low nybble in high position to facilitate output
    call nybout
    retlw 1         ;the character/command has been sent

lcdinit bsf STATUS, RP0
    movlw b'00001110'	;MODIFICATION: added 0 in bit0 for Backlight Control
    andwf TRISB,f           ;RB4-7 outputs
    bcf TRISA,E         ;enable line output
    bcf TRISA,RS        ;rs line output
    bcf STATUS, RP0
    bcf PORTA,RS        ;command mode
    movlw b'00110000'   ;8bit mode
    movwf outbyte
    call nybout
    call qdelay
    call nybout
    call nybout
    bcf outbyte,4       ;4bit mode
    call nybout
    movlw b'00101000'   ;2 lines
    call lcdcmd
    ;movlw b'00001100'   ;disp on,cursor off,blink off
	 movlw b'00001110'   ;disp on,cursor off,blink off
    call lcdcmd
    movlw b'00000110'   ;increasing cursor,no shift
    call lcdcmd
    movlw b'00000001'   ;clear screen- needs delay because it takes 165 clock cycles
    call lcdcmd 
	 clrf cursor
	; decf cursor,f
    call delay  
    retlw 1

delay   decfsz time1,f
    goto delay
    decfsz time2,f
    goto delay
    decf time1,f    ;because they will carry over to 0
    decf time2,f
    return
    
qdelay  decfsz time1,f
    goto qdelay
    decf time1,f
    return

;********************************************************************
;               Serial Routines
;********************************************************************
serialinit bsf STATUS, RP0
	bcf TXSTA, SYNC		;Asynchronous Mode
	bcf TXSTA, BRGH		;Low Clock Speed
	movlw 0x33		;Data Rate(don't just type 51 -it won't work)
	movwf SPBRG
	bcf TXSTA,TX9	;Eight bit send
	bsf TXSTA,TXEN	;enable sending
	bcf STATUS, RP0
	bsf RCSTA,SPEN ;Enable Serial Port
	bsf RCSTA,CREN ;Enable Continuous Receive
	bcf RCSTA,RX9 ;Eight Bit receive
	return
send movwf TXREG
	bsf STATUS,RP0
	btfss TXSTA,TRMT
	goto $-1
	bcf STATUS,RP0
	return
recv btfss PIR1, RCIF
	 goto $-1
	 movf RCREG,w
	 bcf PIR1,RCIF
	 btfss RCSTA,OERR
	 return
	 bcf RCSTA,CREN
	 bsf RCSTA,CREN
	 retlw '?'
;********************************************************************
;               Command Parse	-	tested and working
;********************************************************************
cmd_parse call recv
	movwf test	
	btfss test,4
	goto p4low
		btfsc test,3						
		goto Lcd_Clear
		btfss test,2
		goto pScrl
			btfss test,1
			goto Block_off
				btfss test,0
				goto GPO_off
				goto GPO_on
pScrl 	btfss test,1
			goto Autoscroll_on
				btfss test,0
				goto Autoscroll_off
				goto Block_on
p4low btfss test,3
		goto p3low
			btfss test,2
			goto pULHM
				btfss test,0
				goto Cursor_Left
				goto Cursor_Right
pULHM			btfss test,1
				goto Cursor_Home
					btfss test,0
					goto Underline_on
					goto Underline_off
p3low		btfss test,2
			goto pLWBL
				btfss test,1
				goto Linewrap_off
					btfss test,0
					goto Backlight_off
					goto Set_Cursor
pLWBL			btfss test,0
				goto Backlight_on
				goto Linewrap_on
;********************************************************************
;               Command Functions
;********************************************************************
Left_Routine movlw b'00010000'		;these don't keep track of cursor variable
	call lcdcmd
	call qdelay
	return
Right_Routine movlw b'00010100'
	call lcdcmd
	call qdelay
	return
newline btfss config,6
	goto skip_scroll
	btfss config,5			;skip skipping if on first line
	goto skip_scroll
	movlw b'00000001'		;clear screen (I can't shift)
	call lcdcmd
	clrf cursor
	return
	;???????????????????????? make it shift
skip_scroll	
	bcf cursor,4			;clear the newline test bit(now should be: 00R00000)
	btfss config,7
	return
	call Other_Row
	return
;	movlw b'00011000'
;	movwf nlcount
;nl_loop movlw b'00010100'
;	call lcdcmd
;	call qdelay
;	decfsz nlcount,f
;	goto nl_loop
;	return
Other_Row movlw b'00100000'
	xorwf config,f			;toggle second row status bit
	btfss config,5		;it is already reversed(in dest state)
	goto top_row
	movlw b'11000000'
	call lcdcmd
	return	
top_row
	movlw b'10000000'
	call lcdcmd
	return
;Other_Row movlw b'00101000'	;40 spaces
;	movwf nlcount
;Other_Row_Loop	call Right_Routine
;	decfsz nlcount,f
;	goto Other_Row_Loop
;	movlw b'00100000'
;	xorwf config,f	;toggle row bit
;	return
Update_Cursor	movf config,w
	andlw b'00001111'	;get rid of top status bits
	call lcdcmd
	goto mainloop
Linewrap_on bsf config,7
	goto mainloop
Linewrap_off bcf config,7	
	goto mainloop
Autoscroll_on bsf config,6
	goto mainloop
Autoscroll_off bcf config,6
	goto mainloop
Set_Cursor call recv
		movwf set_col
		call recv
		movwf set_row
		decf set_col,f		;so it is in 0-15 format-worst ever off by 1 error, had this, but going to w
		movlw b'00001111'
		andwf set_col,f	;bounds checking for column (row is automatic)
		movf set_col,w
		subwf cursor,w
		movwf nlcount
		btfsc STATUS,Z
	   goto check_row
		btfsc nlcount,6	;could use 7 here
		goto right_loop
left_loop call Left_Routine
		decfsz nlcount,f
		goto left_loop
		goto check_row
right_loop call Right_Routine
		incfsz nlcount,f
		goto right_loop
		goto check_row
check_row decf set_row,f
		btfss set_row,0
		goto set_row1
			btfss config,5		;to row 2
			call Other_Row
			goto exit_set_cursor
set_row1	btfsc config,5
			call Other_Row
exit_set_cursor movf set_col,w
			movwf cursor		;before we leave, update the cursor variable, row bit is already handled
			goto mainloop
Cursor_Home movlw b'00000010'
	call lcdcmd
	clrf cursor
	bcf config,5		;can consolodate these
	goto mainloop
Underline_on bsf config,1
	goto Update_Cursor
Underline_off bcf config,1
	goto Update_Cursor
Block_on bsf config,0
	goto Update_Cursor
Block_off bcf config,0
	goto Update_Cursor
Cursor_Left movlw b'00010000'
	call lcdcmd
	decf cursor,f				;NEEDS BOUNDS CHECKING
	call qdelay
	goto mainloop
Cursor_Right movlw b'00010100'
	call lcdcmd
	incf cursor,f
	btfsc cursor,4
	call newline
	call qdelay
	goto mainloop
Lcd_Clear movlw b'00000001'
	call lcdcmd
	clrf cursor
	bcf config,5	;can consolodate these
	goto mainloop
Backlight_on bcf PORTB,0
	goto mainloop
Backlight_off bsf PORTB,0
	goto mainloop
GPO_off call recv
	;???????????????????????????
	goto mainloop
GPO_on call recv
	;???????????????????????????
	goto mainloop
;********************************************************************
;               Main
;********************************************************************
start movlw    7 
    movwf CMCON         ; Comparators off, all pins digital I/O 
	 movlw b'10001110'
	 movwf config			;initialize config byte
	 call delay
    call lcdinit
	 call serialinit
	 movlw 'H'
	 call send
	 movlw 'i'
	 call send
    movlw 'R'
    call lcdchar
    movlw 'e'
    call lcdchar
    movlw 'a'
    call lcdchar
    movlw 'd'
    call lcdchar
    movlw 'y'
    call lcdchar
    movlw ' '
    call lcdchar
	 goto Cursor_Home
mainloop call recv
	 movwf test		
	 btfsc test,7
	 goto cmd_parse
	 call lcdchar
	 incf cursor,f	 
	 btfsc cursor,4
	 call newline		 
	 goto mainloop
    END







