
        LIST    p=16F628 ; PIC16F628 is the target processor

    #include "P16F628.INC" ; Include header file
    #define E 0 ;pick any port A pins (0-4)
    #define RS 2

    cblock 0x20   ; Temporary storage
       outbyte
		 config		;bit7:line-wrap,bit6:autoscroll, bit3=1,bit2=1,bit1=Underline,bit0=Block
		 cursor		;bits0-3:column,bit4:newline test,bit5:row
		 nlcount
       time1
       time2
    endc

    __CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _LVP_OFF & _DATA_CP_OFF & _BODEN_OFF 
        org   0

        goto    start

;********************************************************************
;              LCD Routines 
;********************************************************************
;sends high 4 bits of variable 'char' to lcd
nybout  movlw b'11110000'   ;high nybble bitmask
    andwf outbyte,w     ;set top nybble
    iorwf PORTB,f       ;high bits set, now low bits must be set
    movlw b'00001111'   ;high nybble bitmask
    iorwf outbyte,w     ;set top nybble
    andwf PORTB,f       ;low bits set
    bsf PORTA,E
    call qdelay         ;this is necessary, even for a 4MHz clock
    bcf PORTA,E         ;clock it
    retlw 1

;send a character/command in w to lcd
lcdcmd  bcf PORTA,RS        ;RS low for commands
    goto skiprs     ;doing it this way allows the pin to be used for something else
lcdchar bsf PORTA,RS        ;RS high for characters
skiprs  movwf outbyte
    call nybout
    swapf outbyte,f     ;put low nybble in high position to facilitate output
    call nybout
    retlw 1         ;the character/command has been sent

lcdinit bsf STATUS, RP0
    movlw b'00001111'
    andwf TRISB,f           ;RB4-7 outputs
    bcf TRISA,E         ;enable line output
    bcf TRISA,RS        ;rs line output
    bcf STATUS, RP0
    bcf PORTA,RS        ;command mode
    movlw b'00110000'   ;8bit mode
    movwf outbyte
    call nybout
    call qdelay
    call nybout
    call nybout
    bcf outbyte,4       ;4bit mode
    call nybout
    movlw b'00101000'   ;2 lines
    call lcdcmd
    ;movlw b'00001100'   ;disp on,cursor off,blink off
	 movlw b'00001110'   ;disp on,cursor off,blink off
    call lcdcmd
    movlw b'00000110'   ;increasing cursor,no shift
    call lcdcmd
    movlw b'00000001'   ;clear screen- needs delay because it takes 165 clock cycles
    call lcdcmd 
	 clrf cursor
    call delay  
    retlw 1

delay   decfsz time1,f
    goto delay
    decfsz time2,f
    goto delay
    decf time1,f    ;because they will carry over to 0
    decf time2,f
    return
    
qdelay  decfsz time1,f
    goto qdelay
    decf time1,f
    return

;********************************************************************
;               Serial Routines
;********************************************************************
serialinit bsf STATUS, RP0
	bcf TXSTA, SYNC		;Asynchronous Mode
	bcf TXSTA, BRGH		;Low Clock Speed
	movlw 0x33		;Data Rate(don't just type 51 -it won't work)
	movwf SPBRG
	bcf TXSTA,TX9	;Eight bit send
	bsf TXSTA,TXEN	;enable sending
	bcf STATUS, RP0
	bsf RCSTA,SPEN ;Enable Serial Port
	bsf RCSTA,CREN ;Enable Continuous Receive
	bcf RCSTA,RX9 ;Eight Bit receive
	return
send movwf TXREG
	bsf STATUS,RP0
	btfss TXSTA,TRMT
	goto $-1
	bcf STATUS,RP0
	return
recv btfss PIR1, RCIF
	 goto $-1
	 movf RCREG,w
	 bcf PIR1,RCIF
	 return
;********************************************************************
;               Command Parse
;********************************************************************
cmd_parse call recv
	movwf outbyte		;just using this since it's available
	btfss outbyte,4
	goto p4low
		btfsc outbyte,3							;here is parse_4high
		goto Lcd_Clear
		btfss outbyte,2
		goto pScrl
			btfss outbyte,1
			goto Block_off
				btfss outbyte,0
				goto GPO_off
				goto GPO_on
pScrl 	btfss outbyte,1
			goto Autoscroll_on
				btfss outbyte,0
				goto Autoscroll_off
				goto Block_on
p4low btfss outbyte,3
		goto p3low
			btfss outbyte,2
			goto pULHM
				btfss outbyte,0
				goto Cursor_Left
				goto Cursor_Right
pULHM			btfss outbyte,1
				goto Cursor_Home
					btfss outbyte,0
					goto Underline_on
					goto Underline_off
p3low		btfss outbyte,2
			goto pLWBL
				btfss outbyte,1
				goto Linewrap_off
					btfss outbyte,0
					goto Backlight_off
					goto Set_Cursor
pLWBL			btfss outbyte,0
				goto Backlight_on
				goto Linewrap_on
;********************************************************************
;               Command Functions
;********************************************************************
newline btfss config,6
	goto skip_scroll
	btfsc cursor,5
	call Lcd_Clear
	;????????????????????????
skip_scroll	movlw b'00100000'
	xorwf cursor,f			;toggle second row status bit
	bcf cursor,4			;clear the newline test bit(now should be: 00R00000)
	btfss config,7
	return
	movlw b'00011000'
	movwf nlcount
nl_loop call Cursor_Right
	decfsz nlcount,f
	goto nl_loop
	return
Update_Cursor	movf config,w
	andlw b'00001111'	;get rid of top status bits
	call lcdcmd
	goto mainloop
Linewrap_on bsf config,7
	goto mainloop
Linewrap_off bcf config,7	
	goto mainloop
Autoscroll_on bsf config,6
	goto mainloop
Autoscroll_off bcf config,6
	goto mainloop
Set_Cursor call recv
	;????????????????
	call recv
	;????????????????
	goto mainloop
Cursor_Home movlw b'00000010'
	call lcdcmd
	goto mainloop
Underline_on bsf config,1
	goto Update_Cursor
Underline_off bcf config,1
	goto Update_Cursor
Block_on bsf config,0
	goto Update_Cursor
Block_off bcf config,0
	goto Update_Cursor
Cursor_Left movlw b'00010000'
	call lcdcmd
	goto mainloop
Cursor_Right movlw b'00010100'
	call lcdcmd
	goto mainloop
Lcd_Clear movlw b'00000001'
	call lcdcmd
	goto mainloop
Backlight_on bcf PORTB,0
	goto mainloop
Backlight_off bsf PORTB,0
	goto mainloop
GPO_off call recv
	;???????????????????????????
	goto mainloop
GPO_on call recv
	;???????????????????????????
	goto mainloop
;********************************************************************
;               Main
;********************************************************************
start movlw    7 
    movwf CMCON         ; Comparators off, all pins digital I/O 
	 movlw b'00001110'
	 movwf config			;initialize config byte
	 call delay
    call lcdinit
	 call serialinit
	 movlw 'H'
	 call send
	 movlw 'i'
	 call send
    movlw 'R'
    call lcdchar
    movlw 'e'
    call lcdchar
    movlw 'a'
    call lcdchar
    movlw 'd'
    call lcdchar
    movlw 'y'
    call lcdchar
    movlw ' '
    call lcdchar
mainloop call recv
	 movwf outbyte		;just using this since it's available
	 btfsc outbyte,7
	 goto cmd_parse
	 ; call send		;no longer a terminal
	 incf cursor,f
	 movf cursor,w
	 btfsc cursor,4
	 call newline
	 movf outbyte,w
	 call lcdchar
	 goto mainloop
    END







