
        LIST    p=16F628 ; PIC16F628 is the target processor

    #include "P16F628.INC" ; Include header file
    #define E 0 ;pick any port A pins (0-4)
    #define RS 2

    cblock 0x20   ; Temporary storage
       outbyte
       time1
       time2
    endc

    __CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _LVP_OFF & _DATA_CP_OFF & _BODEN_OFF 
        org   0

        goto    start

;********************************************************************
;              LCD Routines 
;********************************************************************
;sends high 4 bits of variable 'char' to lcd
nybout  movlw b'11110000'   ;high nybble bitmask
    andwf outbyte,w     ;set top nybble
    iorwf PORTB,f       ;high bits set, now low bits must be set
    movlw b'00001111'   ;high nybble bitmask
    iorwf outbyte,w     ;set top nybble
    andwf PORTB,f       ;low bits set
    bsf PORTA,E
    call qdelay         ;this is necessary, even for a 4MHz clock
    bcf PORTA,E         ;clock it
    retlw 1

;send a character/command in w to lcd
lcdcmd  bcf PORTA,RS        ;RS low for commands
    goto skiprs     ;doing it this way allows the pin to be used for something else
lcdchar bsf PORTA,RS        ;RS high for characters
skiprs  movwf outbyte
    call nybout
    swapf outbyte,f     ;put low nybble in high position to facilitate output
    call nybout
    retlw 1         ;the character/command has been sent

lcdinit bsf STATUS, RP0
    movlw b'00001111'
    andwf TRISB,f           ;RB4-7 outputs
    bcf TRISA,E         ;enable line output
    bcf TRISA,RS        ;rs line output
    bcf STATUS, RP0
    bcf PORTA,RS        ;command mode
    movlw b'00110000'   ;8bit mode
    movwf outbyte
    call nybout
    call qdelay
    call nybout
    call nybout
    bcf outbyte,4       ;4bit mode
    call nybout
    movlw b'00101000'   ;2 lines
    call lcdcmd
    ;movlw b'00001100'   ;disp on,cursor off,blink off
	 movlw b'00001110'   ;disp on,cursor off,blink off
    call lcdcmd
    movlw b'00000110'   ;increasing cursor,no shift
    call lcdcmd
    movlw b'00000001'   ;clear screen- needs delay because it takes 165 clock cycles
    call lcdcmd 
    call delay  
    retlw 1

delay   decfsz time1,f
    goto delay
    decfsz time2,f
    goto delay
    decf time1,f    ;because they will carry over to 0
    decf time2,f
    return
    
qdelay  decfsz time1,f
    goto qdelay
    decf time1,f
    return

;********************************************************************
;               Serial Routines
;********************************************************************
serialinit bsf STATUS, RP0
	bcf TXSTA, SYNC		;Asynchronous Mode
	bcf TXSTA, BRGH		;Low Clock Speed
	movlw 0x33		;Data Rate(don't just type 51 -it won't work)
	movwf SPBRG
	bcf TXSTA,TX9	;Eight bit send
	bsf TXSTA,TXEN	;enable sending
	bcf STATUS, RP0
	bsf RCSTA,SPEN ;Enable Serial Port
	bsf RCSTA,CREN ;Enable Continuous Receive
	bcf RCSTA,RX9 ;Eight Bit receive

	return
send movwf TXREG
	bsf STATUS,RP0
	btfss TXSTA,TRMT
	goto $-1
	bcf STATUS,RP0
	return
;********************************************************************
;               Main
;********************************************************************
start movlw    7 
    movwf CMCON         ; Comparators off, all pins digital I/O 
	 call delay
    call lcdinit
	 call serialinit
	 movlw 'H'
	 call send
	 movlw 'i'
	 call send
    movlw 'R'
    call lcdchar
    movlw 'e'
    call lcdchar
    movlw 'a'
    call lcdchar
    movlw 'd'
    call lcdchar
    movlw 'y'
    call lcdchar
    movlw ' '
    call lcdchar
check btfss PIR1, RCIF
	 goto $-1
	 movf RCREG,w
	 bcf PIR1,RCIF
	 call send
	 call lcdchar
	 goto check
    END







